<?php
	session_start();

	// if new session, reset session variable 'attempts'
	// represents number of attempts
	if (!isset($_SESSION['attempts'])) {
		$_SESSION['attempts'] = 0;
	}

	// constants 
	define('red_dot', '<span class="dot red"></span>');
	define('black_dot', '<span class="dot black"></span>');
	define('white_dot', '<span class="dot white"></span>');

	// if no code generated yet..
	if (!isset($_SESSION['code'])) {

		// ..create array for code chars
		$code_array = array();
		// is code already generated?
		$code_generated = false;

		/*
		*	..create random character and add to 
		*	$code_array if not already existing. 
		*	repeat until code has 4 characters.
		*/
		while (!$code_generated) {
			$char = createRandomChar();

			/* 
			*	trial and error -
			* 	would probably be faster to provide array with 
			* 	available characters and pop out the used ones (next time..)
			*/
			if (!in_array($char, $code_array)) {
				array_push($code_array, $char);
			}
			
			if (sizeof($code_array) == 4) {
				$code_generated = true;

				// ..set generated code as session variable
				$_SESSION['code'] = $code_array;
			}
		}
	}

	

	// create and return random character
	function createRandomChar() {
		$rand_nr = rand(1,7);
		$char = null;

		switch ($rand_nr) {
		    case 1:
		    	$char = 'A';
		        break;
		    case 2:
		    	$char = 'B';
		        break;
		    case 3:
		    	$char = 'C';
		        break;
		    case 4:
		    	$char = 'D';
		        break;
		    case 5:
		    	$char = 'E';
		        break;
		    case 6:
		    	$char = 'F';
		        break;
		    case 7:
		    	$char = 'G';
		        break;
		}

		return $char;
	}

	// validate input (= check if length is 1. pre-validation happens via input pattern)
	function isInputValid($array) {
		foreach ($array as $value) {
			$valid = false;

			if (strlen($value) == 1) {
				$valid = true;
			} else {
				$valid = false;
			}			
		}

		return $valid;
	}

	function hasDuplicates($array) {
 		$duplicateArray = array();

 		foreach($array as $val){
  			if(++$duplicateArray[$val] > 1){
   				return true;
  			}
 		}

		return false;
	}

?>
<html>
	<head>
		<title>Codebreaker</title>
		<link rel="stylesheet" href="stylesheets/style.css">
		<meta charset="UTF-8">
  		<meta name="viewport" content="width=device-width, initial-scale=1; user-scalable=no">
    	<meta http-equiv="X-UA-Compatible" content="IE=edge,chrome=1">
	</head>
	<body>
		<div class="game-container">
			<h1>Codebreaker</h1>

			<form action="index.php" method="post">
				<input type="text" name="input_a" maxlength="1" size="1" pattern="[A-Ga-g]" id="input_1" required>
				<input type="text" name="input_b" maxlength="1" size="1" pattern="[A-Ga-g]" id="input_2" required>
				<input type="text" name="input_c" maxlength="1" size="1" pattern="[A-Ga-g]" id="input_3" required>
				<input type="text" name="input_d" maxlength="1" size="1" pattern="[A-Ga-g]" id="input_4" required>
				<input type="submit" name="submit" value="Check">
			</form>
			<a href="index.php?reset=true">Restart</a>
			
			<?php 

				// delete session -> restart game
				if ($_GET['reset']) {
					session_unset();
				}	

				if(isset($_POST['submit'])) {
			?>

				<div class="guess-history">
					
				<?php 
					/* 
					* 	session variable 'attempted_codes' contains
					* 	each character of every attempt as an array
					*/
					if (!isset($_SESSION['attempted_codes'])) {
						$_SESSION['attempted_codes'] = array();
					}

					// array with single input chars
					$code_input = array(
							strtoupper($_POST['input_a']), 
							strtoupper($_POST['input_b']), 
							strtoupper($_POST['input_c']), 
							strtoupper($_POST['input_d'])
						);

					// only if input is valid, push entered code to attempts
					if (isInputValid($code_input) && !hasDuplicates($code_input)) {
						array_push($_SESSION['attempted_codes'], $code_input);

						// increase number of attempts
						$_SESSION['attempts']++;
					} else {
						?>
							<script>
								alert('Ungueltige Eingabe! Buchstaben duerfen nicht mehrfach vorkommen.');
							</script>
						<?php  
					}

					// count the correct, semi-correct and wrong guesses
					$correct;	// -> character right, position right.
					$semi_correct; // -> character right, position wrong.
					$wrong;	// -> all wrong :(
					
					echo "<div class='attempts'> Versuch Nr. " . $_SESSION['attempts'] . '/10</div>';

					// iterate through all entered coded..
					foreach ($_SESSION['attempted_codes'] as $code) {
						// .. and display them
						echo "<div>";
						echo implode(" ", $code)." ";
							
							// reset values
							$correct = 0;	
							$semi_correct = 0;
							$wrong = 0;

						// iterate through each character in current code
						foreach ($code as $key => $code_char) {


							

							// if the code contains current character..
							if (in_array($code_char, $_SESSION['code'])) {
								// ..check if the position is right
								if (strcmp($_SESSION['code'][$key], $code_char) == 0) {
									// right? increase number of correct chars
									$correct++;
								} else {
									// wrong position?
									$semi_correct++;
								}
							} else {
								// code does not contain entered character.
								$wrong++;
							}
						}

						// draw dots.
						for ($i = 0; $i < $semi_correct; $i++) {
							echo black_dot; 
						}

						for ($i = 0; $i < $correct; $i++) {
							echo red_dot;
						} 

						for ($i = 0; $i < $wrong; $i++) {
							echo white_dot;
						}

						// code as string
						$code = implode("", $_SESSION['code']);

						/* if attempts >= 10, check if solution is right or not and alert message.
							if attemps < 10 just check if solution is right
						*/
						if ($_SESSION['attempts'] >= 10) {
							if ($correct == 4) {
								?>
									<script>
										alert('Gewonnen! Der Code war  <?php echo $code ?>');
										window.document.location.href = "index.php";
									</script>
								<?php  

							} else {
								?>
									<script>
										alert('Game Over! Der Code war <?php echo $code ?>');
										window.document.location.href = "index.php";
									</script>
								<?php 
							}
						} else if ($correct == 4) {
							?>
								<script>
									alert('Gewonnen! Der Code war <?php echo $code ?>');
									window.document.location.href = "index.php";
								</script>
							<?php 						
						}

						echo "</div>";
					}

	        	?>

				</div>
			
			<?php 
				}
			?>
		    
			
			

		</div>
	</body>
</html>